"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupWindowsEnvironmentPath = void 0;
const path_1 = require("path");
const execa_1 = __importDefault(require("execa"));
const string_prototype_matchall_1 = __importDefault(require("string.prototype.matchall"));
const BadHomeDirError_1 = require("./BadHomeDirError");
const REG_KEY = 'HKEY_CURRENT_USER\\Environment';
function findEnvValuesInRegistry(regEntries, envVarName) {
    const regexp = new RegExp(`^ {4}(?<name>${envVarName}) {4}(?<type>\\w+) {4}(?<data>.*)$`, 'gim');
    return Array.from((0, string_prototype_matchall_1.default)(regEntries, regexp));
}
function setEnvVarInRegistry(envVarName, envVarValue) {
    // `windowsHide` in `execa` is true by default, which will cause `chcp` to have no effect.
    return (0, execa_1.default)('reg', ['add', REG_KEY, '/v', envVarName, '/t', 'REG_EXPAND_SZ', '/d', envVarValue, '/f'], { windowsHide: false });
}
function pathIncludesDir(pathValue, dir) {
    const dirPath = path_1.win32.parse(path_1.win32.normalize(dir));
    return pathValue
        .split(path_1.win32.delimiter)
        .map(p => path_1.win32.normalize(p))
        .map(p => path_1.win32.parse(p))
        .map(p => `${p.dir}${path_1.win32.sep}${p.base}`.toUpperCase())
        .filter(p => p !== '')
        .includes(`${dirPath.dir}${path_1.win32.sep}${dirPath.base}`.toUpperCase());
}
async function setupWindowsEnvironmentPath(pnpmHomeDir, opts) {
    var _a;
    // Use `chcp` to make `reg` use utf8 encoding for output.
    // Otherwise, the non-ascii characters in the environment variables will become garbled characters.
    const chcpResult = await (0, execa_1.default)('chcp');
    const cpMatch = (_a = /\d+/.exec(chcpResult.stdout)) !== null && _a !== void 0 ? _a : [];
    const cpBak = parseInt(cpMatch[0]);
    if (chcpResult.failed || !(cpBak > 0)) {
        return `exec chcp failed: ${cpBak}, ${chcpResult.stderr}`;
    }
    await (0, execa_1.default)('chcp', ['65001']);
    try {
        return await _setupWindowsEnvironmentPath(path_1.win32.normalize(pnpmHomeDir), opts);
    }
    finally {
        await (0, execa_1.default)('chcp', [cpBak.toString()]);
    }
}
exports.setupWindowsEnvironmentPath = setupWindowsEnvironmentPath;
async function _setupWindowsEnvironmentPath(pnpmHomeDir, opts) {
    var _a;
    const queryResult = await (0, execa_1.default)('reg', ['query', REG_KEY], { windowsHide: false });
    if (queryResult.failed) {
        return 'Win32 registry environment values could not be retrieved';
    }
    const queryOutput = queryResult.stdout;
    const pathValueMatch = findEnvValuesInRegistry(queryOutput, 'PATH');
    const homeValueMatch = findEnvValuesInRegistry(queryOutput, 'PNPM_HOME');
    let commitNeeded = false;
    const logger = [];
    if (homeValueMatch.length === 1 && !opts.force) {
        const currentHomeDir = homeValueMatch[0].groups.data;
        if (currentHomeDir !== pnpmHomeDir) {
            throw new BadHomeDirError_1.BadHomeDirError({ currentDir: currentHomeDir, wantedDir: pnpmHomeDir });
        }
    }
    else {
        logger.push(`Setting 'PNPM_HOME' to value '${pnpmHomeDir}'`);
        const addResult = await setEnvVarInRegistry('PNPM_HOME', pnpmHomeDir);
        if (addResult.failed) {
            logger.push(`\t${addResult.stderr}`);
        }
        else {
            commitNeeded = true;
            logger.push(`\t${addResult.stdout}`);
        }
    }
    const pathData = (_a = pathValueMatch[0]) === null || _a === void 0 ? void 0 : _a.groups.data;
    if (pathData === undefined) {
        logger.push('Current PATH is not set. No changes to this environment variable are applied');
    }
    else if (pathData == null || pathData.trim() === '') {
        logger.push('Current PATH is empty. No changes to this environment variable are applied');
    }
    else if (pathIncludesDir(pathData, pnpmHomeDir)) {
        logger.push('PATH already contains PNPM_HOME');
    }
    else {
        logger.push('Updating PATH');
        const newPathValue = `${pnpmHomeDir}${path_1.win32.delimiter}${pathData}`;
        const addResult = await setEnvVarInRegistry(pathValueMatch[0].groups.name, newPathValue);
        if (addResult.failed) {
            logger.push(`\t${addResult.stderr}`);
        }
        else {
            commitNeeded = true;
            logger.push(`\t${addResult.stdout}`);
        }
    }
    if (commitNeeded) {
        await (0, execa_1.default)('setx', ['PNPM_HOME', pnpmHomeDir]);
    }
    return logger.join('\n');
}
//# sourceMappingURL=setupOnWindows.js.map